@@ -17,30 +17,18 @@ bash miniconda.sh -b -p $MINICONDA
 export PATH="$MINICONDA/bin:$PATH"
 
 conda config --set auto_update_conda false
-conda config --add channels https://repo.continuum.io/pkgs/free
 conda config --add channels conda-forge
+conda install -y mamba yq jq
 
 if [ "${USE_DEV_WHEELS}" = "nightlies" ]; then
-    export CONDA_ARROW="arrow-nightlies::pyarrow arrow-nightlies::arrow-cpp -c arrow-nightlies"
+    export CUSTOM_CONDA_CHANNELS='"arrow-nightlies", "conda-forge"'
 else
-    export CONDA_ARROW="pyarrow arrow-cpp"
+    export CUSTOM_CONDA_CHANNELS='"conda-forge"'
 fi
 
-conda create -y -q -n fletcher python=${PYTHON_VERSION} \
-    pandas pytest pytest-cov \
-    hypothesis \
-    setuptools_scm \
-    pip \
-    numba \
-    codecov \
-    six \
-    sphinx \
-    sphinx_rtd_theme \
-    numpydoc \
-    sphinxcontrib-apidoc \
-    pre_commit \
-    $CONDA_ARROW \
-    -c conda-forge
+yq -Y ". + {channels: [${CONDA_CHANNELS}], dependencies: [.dependencies[], \"python=${PYTHON_VERSION}\"] }" environment.yml > /tmp/environment.yml
+cat /tmp/environment.yml
+mamba env create -f /tmp/environment.yml
 source activate fletcher
 
 if [ "${PYTHON_VERSION}" = "3.7" ]; then
@@ -55,7 +43,7 @@ if [ "${USE_DEV_WHEELS}" = "nightlies" ]; then
     pip install --pre --no-deps --upgrade --timeout=60 -i $PRE_WHEELS pandas
 fi
 
-pip install --no-deps -e .
+python -m pip install -e . --no-build-isolation --no-use-pep517
 py.test --junitxml=test-reports/junit.xml --cov=./ --cov-report=xml
 
 # Do a second run with JIT disabled to produce coverage and check that the
@@ -65,9 +53,12 @@ if [ "${PYTHON_VERSION}" = "3.6" ]; then
   NUMBA_DISABLE_JIT=1 py.test --junitxml=test-reports/junit.xml --cov=./ --cov-report=xml
 fi
 
-# Check documentation build only in one job
+# Check documentation build only in one job, also do releases
 if [ "${PYTHON_VERSION}" = "3.7" ]; then
   pushd docs
   make html
   popd
+
+  python setup.py sdist
+  python setup.py bdist_wheel
 fi
