@@ -20,7 +20,6 @@ from pandas.core.arrays import ExtensionArray
 from pandas.core.dtypes.dtypes import ExtensionDtype, register_extension_dtype
 
 from fletcher._algorithms import (
-    extract_isnull_bytemap,
     kurt_op,
     max_op,
     median_op,
@@ -39,6 +38,8 @@ PANDAS_GE_0_26_0 = LooseVersion(pd.__version__) >= "0.26.0"
 if PANDAS_GE_0_26_0:
     from pandas.core.indexers import check_array_indexer
 
+ARROW_GE_0_18_0 = LooseVersion(pa.__version__) >= "0.18.0"
+
 _python_type_map = {
     pa.null().id: str,
     pa.bool_().id: bool,
@@ -60,11 +61,67 @@ _python_type_map = {
     pa.string().id: str,
     # Use any list type here, only LIST is important
     pa.list_(pa.string()).id: list,
+    # Use any large list type here, only LIST is important
+    pa.large_list(pa.string()).id: list,
+    # Use any dictionary type here, only dict is important
+    pa.dictionary(pa.int32(), pa.int32()).id: dict,
     pa.duration("ns").id: datetime.timedelta,
 }
 
 _string_type_map = {"date64[ms]": pa.date64(), "string": pa.string()}
 
+_examples = {
+    pa.null(): pa.array([None, None], type=pa.null()),
+    pa.bool_(): pa.array([None, True], type=pa.bool_()),
+    pa.int8(): pa.array([None, -1], type=pa.int8()),
+    pa.uint8(): pa.array([None, 1], type=pa.uint8()),
+    pa.int16(): pa.array([None, -1], type=pa.int16()),
+    pa.uint16(): pa.array([None, 1], type=pa.uint16()),
+    pa.int32(): pa.array([None, -1], type=pa.int32()),
+    pa.uint32(): pa.array([None, 1], type=pa.uint32()),
+    pa.int64(): pa.array([None, -1], type=pa.int64()),
+    pa.uint64(): pa.array([None, 1], type=pa.uint64()),
+    pa.float16(): pa.array([None, np.float16(-0.1)], type=pa.float16()),
+    pa.float32(): pa.array([None, -0.1], type=pa.float32()),
+    pa.float64(): pa.array([None, -0.1], type=pa.float64()),
+    pa.date32(): pa.array([None, datetime.date(2010, 9, 8)], type=pa.date32()),
+    pa.date64(): pa.array([None, datetime.date(2010, 9, 8)], type=pa.date64()),
+    pa.timestamp("s"): pa.array(
+        [None, datetime.datetime(2013, 12, 11, 10, 9, 8)], type=pa.timestamp("s")
+    ),
+    pa.timestamp("ms"): pa.array(
+        [None, datetime.datetime(2013, 12, 11, 10, 9, 8, 1000)], type=pa.timestamp("ms")
+    ),
+    pa.timestamp("us"): pa.array(
+        [None, datetime.datetime(2013, 12, 11, 10, 9, 8, 7)], type=pa.timestamp("us")
+    ),
+    pa.timestamp("ns"): pa.array(
+        [None, datetime.datetime(2013, 12, 11, 10, 9, 8, 7)], type=pa.timestamp("ns")
+    ),
+    pa.binary(): pa.array([None, b"122"], type=pa.binary()),
+    pa.string(): pa.array([None, "🤔"], type=pa.string()),
+    pa.duration("s"): pa.array(
+        [None, datetime.timedelta(seconds=9)], type=pa.duration("s")
+    ),
+    pa.duration("ms"): pa.array(
+        [None, datetime.timedelta(milliseconds=8)], type=pa.duration("ms")
+    ),
+    pa.duration("us"): pa.array(
+        [None, datetime.timedelta(microseconds=7)], type=pa.duration("us")
+    ),
+    pa.duration("ns"): pa.array(
+        [None, datetime.timedelta(microseconds=7)], type=pa.duration("ns")
+    ),
+}
+
+
+def _get_example(arrow_dtype: pa.DataType) -> pa.Array:
+    if isinstance(arrow_dtype, pa.ListType):
+        return pa.array(
+            [None, _get_example(arrow_dtype.value_type).to_pylist()], type=arrow_dtype
+        )
+    return _examples[arrow_dtype]
+
 
 def _is_numeric(arrow_dtype: pa.DataType) -> bool:
     return (
@@ -130,6 +187,8 @@ class FletcherBaseDtype(ExtensionDtype):
         """
         if pa.types.is_date(self.arrow_dtype):
             return "O"
+        elif self._is_list:
+            return "O"
         else:
             return np.dtype(self.arrow_dtype.to_pandas_dtype()).kind
 
@@ -149,10 +208,20 @@ class FletcherBaseDtype(ExtensionDtype):
     def _is_numeric(self):
         return _is_numeric(self.arrow_dtype)
 
+    @property
+    def _is_list(self):
+        return pa.types.is_list(self.arrow_dtype) or pa.types.is_large_list(
+            self.arrow_dtype
+        )
+
     def __from_arrow__(self, data):
         """Construct a FletcherArray from an arrow array."""
         return self.construct_array_type()(data)
 
+    def example(self):
+        """Get a simple array with example content."""
+        return self.construct_array_type()(_get_example(self.arrow_dtype))
+
 
 @register_extension_dtype
 class FletcherContinuousDtype(FletcherBaseDtype):
@@ -362,14 +431,6 @@ class FletcherBaseArray(ExtensionArray):
         """
         return self.shape[0]
 
-    def isna(self) -> np.ndarray:
-        """
-        Boolean NumPy array indicating if each value is missing.
-
-        This should return a 1-D array the same length as 'self'.
-        """
-        return extract_isnull_bytemap(self.data)
-
     @property
     def base(self) -> Union[pa.Array, pa.ChunkedArray]:
         """Return base object of the underlying data."""
@@ -461,6 +522,13 @@ class FletcherBaseArray(ExtensionArray):
                 raise NotImplementedError(
                     f"Only method == '__call__' is supported in ufuncs, not '{method}'"
                 )
+        if len(inputs) == 1:
+            if getattr(ufunc, "__name__") == "isnan":
+                return self.isna()
+            else:
+                raise NotImplementedError(
+                    f"ufunc with single input not supported: {ufunc}"
+                )
         if len(inputs) != 2:
             raise NotImplementedError("Only ufuncs with a second input are supported")
         if len(kwargs) > 0:
@@ -477,7 +545,7 @@ class FletcherBaseArray(ExtensionArray):
 
     def _np_ufunc_op(self, op: Callable, other):
         """Apply a NumPy ufunc on the instance and any other object."""
-        if isinstance(other, pd.Series):
+        if isinstance(other, (pd.Series, pd.DataFrame)):
             return NotImplemented
         if isinstance(other, FletcherBaseArray):
             other = other.data
@@ -485,7 +553,7 @@ class FletcherBaseArray(ExtensionArray):
 
     def _np_compare_op(self, op: Callable, np_op: Callable, other):
         """Apply a NumPy-based comparison on the instance and any other object."""
-        if isinstance(other, pd.Series):
+        if isinstance(other, (pd.Series, pd.DataFrame)):
             return NotImplemented
         # TODO: Only numeric comparisons are fast currently
         if not self.dtype._is_numeric:
@@ -728,7 +796,7 @@ class FletcherBaseArray(ExtensionArray):
 
         # NumPy's conversion of list->unicode is differently from Python's
         # default. We want to have the default Python output, so force it here.
-        if pa.types.is_list(self.dtype.arrow_dtype) and dtype.kind == "U":
+        if (self.dtype._is_list) and dtype.kind == "U":
             result = np.array([str(x) for x in self.data.to_pylist()])
             if pandas_type is not None:
                 return pd.array(result, dtype=pandas_type)
@@ -773,6 +841,14 @@ class FletcherBaseArray(ExtensionArray):
 
         return pd.Series(counts, index=index)
 
+    def isna(self) -> np.ndarray:
+        """
+        Boolean NumPy array indicating if each value is missing.
+
+        This should return a 1-D array the same length as 'self'.
+        """
+        return np.array(self.data.is_null())
+
 
 class FletcherContinuousArray(FletcherBaseArray):
     """Pandas ExtensionArray implementation backed by Apache Arrow's pyarrow.Array."""
@@ -839,7 +915,7 @@ class FletcherContinuousArray(FletcherBaseArray):
         if PANDAS_GE_0_26_0:
             key = check_array_indexer(self, key)
 
-        if pa.types.is_list(self.data.type):
+        if self.dtype._is_list:
             # TODO: We can probably implement this for the scalar case?
             # TODO: Implement a list accessor and then the three mentioned methods
             raise ValueError(
@@ -915,9 +991,8 @@ class FletcherContinuousArray(FletcherBaseArray):
         if PANDAS_GE_0_26_0:
             item = check_array_indexer(self, item)
 
-        # Workaround for Arrow bug that segfaults on empty slice.
-        # This is fixed in Arrow master, will be released in 0.10
-        if isinstance(item, slice):
+        # Arrow 0.18+ supports slices perfectly
+        if isinstance(item, slice) and not ARROW_GE_0_18_0:
             start = item.start or 0
             stop = item.stop if item.stop is not None else len(self.data)
             stop = min(stop, len(self.data))
@@ -950,6 +1025,7 @@ class FletcherContinuousArray(FletcherBaseArray):
                     "Only integers, slices and integer or boolean arrays are valid indices."
                 )
         elif is_integer(item):
+            item = int(item)
             if item < 0:
                 item += len(self)
             if item >= len(self):
@@ -1158,6 +1234,12 @@ class FletcherContinuousArray(FletcherBaseArray):
         """
         return self._take_array(self.data, indices, allow_fill, fill_value)
 
+    def flatten(self):
+        """
+        Flatten the array.
+        """
+        return type(self)(self.data.flatten())
+
 
 class FletcherChunkedArray(FletcherBaseArray):
     """Pandas ExtensionArray implementation backed by Apache Arrow."""
@@ -1241,7 +1323,7 @@ class FletcherChunkedArray(FletcherBaseArray):
         if PANDAS_GE_0_26_0:
             key = check_array_indexer(self, key)
 
-        if pa.types.is_list(self.data.type):
+        if self.dtype._is_list:
             # TODO: We can probably implement this for the scalar case?
             # TODO: Implement a list accessor and then the three mentioned methods
             raise ValueError(
@@ -1334,9 +1416,8 @@ class FletcherChunkedArray(FletcherBaseArray):
         if PANDAS_GE_0_26_0:
             item = check_array_indexer(self, item)
 
-        # Workaround for Arrow bug that segfaults on empty slice.
-        # This is fixed in Arrow master, will be released in 0.10
-        if isinstance(item, slice):
+        # Arrow 0.18+ supports slices perfectly
+        if isinstance(item, slice) and not ARROW_GE_0_18_0:
             start = item.start or 0
             stop = item.stop if item.stop is not None else len(self.data)
             stop = min(stop, len(self.data))
@@ -1369,6 +1450,7 @@ class FletcherChunkedArray(FletcherBaseArray):
                     "Only integers, slices and integer or boolean arrays are valid indices."
                 )
         elif is_integer(item):
+            item = int(item)
             if item < 0:
                 item += len(self)
             if item >= len(self):
@@ -1595,6 +1677,14 @@ class FletcherChunkedArray(FletcherBaseArray):
         result = take(data, indices, fill_value=fill_value, allow_fill=allow_fill)
         return self._from_sequence(result, dtype=self.data.type)
 
+    def flatten(self):
+        """
+        Flatten the array.
+        """
+        return type(self)(
+            pa.chunked_array(ch.flatten() for ch in self.data.iterchunks())
+        )
+
 
 def pandas_from_arrow(
     arrow_object: Union[pa.RecordBatch, pa.Table, pa.Array, pa.ChunkedArray],
